const Type = [
        "Rape",
        "Kidnapping and Abduction",
        "Dowry Deaths",
        "Assault on Women",
        "Insult to Modesty of Women",
        "Cruelty by Husband or His Relatives",
      ].sort();

      //color theme
      const ColorTheme = {
        Rape: ["#AF0225", "#FDC9B4"],
        "Kidnapping and Abduction": ["#036429", "#D3EECD"],
        "Dowry Deaths": ["#01665E", "#E2E2E2"],
        "Assault on Women": ["#0A4A90", "#CFE1F2"],
        "Insult to Modesty of Women": ["#700174", "#CCDDEC"],
        "Cruelty by Husband or His Relatives": ["#8F3204", "#FEEAA1"],
      };

      //data
      const csvUrl = [
        "https://gist.githubusercontent.com/",
        "lzlazmSylas/", // User
        "c5d21ca1a87fefbe69f6b927a035db58/", // Id of the Gist
        "raw/bfec217e6af338a901af214f2ce4772026b4e6d9/", // commit
        "Crime%2520Against%2520to%2520Indian%2520Woman%2520Ver1.3.csv", // File name
      ].join("");

      //map shape json
      const topojsonUrl =
        "https://raw.githubusercontent.com/deldersveld/topojson/master/countries/india/india-states.json";

      fetchData().then((dataset) => {
        //all data
        const data = parseData(dataset[0]); //state level data
        const india = topojson.feature(
          dataset[1],
          dataset[1].objects["IND_adm1"]
        ).features; //get map shape geojson

        //chart container
        const mainChartContainer = d3.select(".main-chart");
        const donut0Container = d3.select(".donut0");
        const donut1Container = d3.select(".donut1");

        //dropdown
        const dropdownSelected = { year: 2006, type: Type[0] }; //default value

        
        

        //donut0  dropdown - year
        donut0Container
          .append("select")
          .call((s) => {
            s.selectAll("option")
              .data([...new Set(data.map((d) => d.year))].sort())
              .join("option")
              .attr("value", (d) => d)
              .property("selected", (d) => d == dropdownSelected.year)
              .text((d) => d);
          })
          .on("change", (event) => {
            const year = event.target.value;
            donut0(donut0Container, data, year);
          });

        //
        //dropdown - type
        donut1Container
          .append("select")
          .call((s) => {
            s.selectAll("option")
              .data(Type)
              .join("option")
              .attr("value", (d) => d)
              .property("selected", (d) => d == dropdownSelected.type)
              .text((d) => d);
          })
          .on("change", (event) => {
            const type = event.target.value;
            donut1(donut1Container, data, type);
          });

        //init, using default data
        const chosenData = updateMainChartData(data, india, dropdownSelected);
        mainChart(mainChartContainer, chosenData, dropdownSelected.type);

        donut0(donut0Container, data, dropdownSelected.year);
        donut1(donut1Container, data, dropdownSelected.type);
      });

      function donut0(container, dataset, optionYear) {
        //data
        const data = d3
          .rollups(
            dataset
              .filter((d) => d.year == optionYear)
              .map((d) => d.data)
              .flat(),
            (v) => d3.sum(v, (d) => d.value),
            (d) => d.type
          )
          .map((d) => ({ type: d[0], value: d[1] }));

        const totalValue = d3.sum(data, (d) => d.value);

        const innerRadius = 80;
        const outerRadius = 120;
        const innerWidth = 400;
        const innerHeight = 250;
        const margin = { top: 0, right: 0, bottom: 0, left: 0 };

        const width = innerWidth + margin.left + margin.right;
        const height = innerHeight + margin.top + margin.bottom;

        //
        const color = d3
          .scaleOrdinal()
          .domain(Object.keys(ColorTheme))
          .range(
            Object.values(ColorTheme).map((d) => d3.scaleLinear().range(d)(0.3))
          );

        //
        const svg = container
          .selectAll("svg")
          .data(["svg"])
          .join("svg")
          .attr("width", "100%")
          .attr("viewBox", [0, 0, width, height])
          .attr("overflow", "visible");

        svg
          .selectAll("g.inner")
          .data(["inner"])
          .join("g")
          .attr("class", "inner")
          .attr(
            "transform",
            `translate(${margin.left + innerWidth / 2},${
              margin.top + innerHeight / 2
            })`
          )
          .call(markPie);

        function markPie(g) {
          const arcGenerator = d3
            .arc()
            .startAngle((d) => d.startAngle)
            .endAngle((d) => d.endAngle)
            .innerRadius(innerRadius)
            .outerRadius(outerRadius);

          const pieData = d3.pie().value((d) => d.value)(data);
          const pie = g.selectAll("path").data(pieData);

          pie
            .enter()
            .append("path")
            .attr("stroke", "none")
            .attr("stroke-width", "none")
            .attr("fill", (d) => color(d.data.type))
            .attr("d", arcGenerator);

          pie
            .transition()
            .attr("fill", (d) => color(d.data.type))
            .attr("d", arcGenerator)
            .on("interrupt cancel", function (d) {
              d3.select(this)
                .attr("fill", color(d.data.type))
                .attr("d", arcGenerator);
            });

          pie.exit().remove();

          g.selectAll("text")
            .data(pieData)
            .join("text")

            .attr("text-anchor", (d) => {
              d.midAngle = (d.startAngle + d.endAngle) / 2;
              return d.midAngle <= Math.PI ? "start" : "end";
            })
            .attr("dominant-baseline", (d) =>
              d.midAngle >= Math.PI / 2 && d.midAngle <= (Math.PI * 3) / 2
                ? "hanging"
                : "auto"
            )
            .attr("fill", (d) => color(d.data.type))
            .attr("font-size", 8)
            .attr("x", (d) => outerRadius * 1.05 * Math.sin(d.midAngle))
            .attr("y", (d) => outerRadius * 1.05 * Math.cos(d.midAngle) * -1)
            .text((d) =>
              d.data.type == "Cruelty by Husband or His Relatives"
                ? "Cruelty by Husband" + "\n" + "or His Relatives"
                : d.data.type
            );

          const path = g.selectAll("path");
          path
            .selectAll("title")
            .data((d) => [d])
            .join("title")
            .text(
              (d) =>
                d.data.type +
                "\n" +
                Math.round((1000 * d.data.value) / totalValue) / 10 +
                "%"
            );

          path
            .on("mouseover", function (e, dd) {
              path
                .filter((d) => d.index !== dd.index)
                .transition()
                .attrTween("fill", function () {
                  const prev = d3.select(this).attr("fill");
                  return d3.interpolate(prev, "#E8E8E8");
                })
                .on("interrupt cancel", function () {
                  d3.select(this).attr("fill", "#E8E8E8");
                });

              path
                .filter((d) => d.index == dd.index)
                .transition()
                .attrTween("fill", function (d) {
                  const prev = d3.select(this).attr("fill");
                  return d3.interpolate(prev, color(d.data.type));
                })
                .on("interrupt cancel", function (d) {
                  d3.select(this).attr("fill", (d) => color(d.data.type));
                });
            })
            .on("mouseout", function (e, dd) {
              path
                .transition()
                .attrTween("fill", function (d) {
                  const prev = d3.select(this).attr("fill");
                  return d3.interpolate(prev, color(d.data.type));
                })
                .on("interrupt cancel", function (d) {
                  d3.select(this).attr("fill", (d) => color(d.data.type));
                });
            });
        }
      }

      var donut1Color;
      function donut1(container, dataset, optionType) {
        //data
        const data = d3
          .rollups(
            dataset.map((d) => {
              const found = d.data.filter((d) => d.type == optionType)[0];
              return { ...d, type: found.type, value: found.value };
            }),
            (v) => d3.sum(v, (d) => d.value),
            (d) => d.state
          )
          .map((d) => ({ state: d[0], value: d[1] }));

        const totalValue = d3.sum(data, (d) => d.value);

        const innerRadius = 80;
        const outerRadius = 120;
        const innerWidth = 400;
        const innerHeight = 250;
        const margin = { top: 0, right: 0, bottom: 0, left: 0 };

        const width = innerWidth + margin.left + margin.right;
        const height = innerHeight + margin.top + margin.bottom;

        //
        const color = d3
          .scaleLinear()
          .domain(d3.extent(data, (d) => d.value).reverse())
          .range(ColorTheme[optionType]);

        donut1Color = color;
        //
        const svg = container
          .selectAll("svg")
          .data(["svg"])
          .join("svg")
          .attr("width", "100%")
          .attr("viewBox", [0, 0, width, height]);

        svg
          .selectAll("g.inner")
          .data(["inner"])
          .join("g")
          .attr("class", "inner")
          .attr(
            "transform",
            `translate(${margin.left + innerWidth / 2},${
              margin.top + innerHeight / 2
            })`
          )
          .call(markPie);

        function markPie(g) {
          const arcGenerator = d3
            .arc()
            .startAngle((d) => d.startAngle)
            .endAngle((d) => d.endAngle)
            .innerRadius(innerRadius)
            .outerRadius(outerRadius);

          const pie = g
            .selectAll("path")
            .data(d3.pie().value((d) => d.value)(data), (d) => d.data.state);

          pie
            .enter()
            .append("path")
            .attr("stroke", "none")
            .attr("stroke-width", "none")
            .attr("fill", (d) => color(d.value))
            .attr("d", arcGenerator);

          pie
            .transition()
            .attr("fill", (d) => color(d.value))
            .attr("d", arcGenerator)
            .on("interrupt cancel", function (d) {
              d3.select(this)
                .attr("fill", color(d.data.type))
                .attr("d", arcGenerator);
            });

          pie.exit().remove();

          const path = g.selectAll("path");
          path
            .selectAll("title")
            .data((d) => [d])
            .join("title")
            .text(
              (d) =>
                d.data.state +
                "\n" +
                Math.round((1000 * d.value) / totalValue) / 10 +
                "%"
            );

          path
            .on("mouseover", function (e, dd) {
              stateHighlight(dd.data.state);
            })
            .on("mouseout", function (e, dd) {
              stateRecover();
            });
        }
      }

      var mainChartColor;
      function mainChart(container, data, optionType) {
        //
        const mapWidth = 600;
        const mapHeight = mapWidth;

        const barChartWidth = 200;

        const innerWidth = mapWidth + barChartWidth;
        const innerHeight = mapHeight;
        const margin = { top: 20, right: 20, bottom: 20, left: 150 };

        const width = innerWidth + margin.left + margin.right;
        const height = innerHeight + margin.top + margin.bottom;

        //scale
        const xScale = d3
          .scaleLinear()
          .domain([0, d3.max(data, (d) => d.data.value)])
          .nice()
          .range([0, barChartWidth]); //bar x axis scale

        const yScale = d3
          .scaleBand()
          .domain(data.map((d) => d.data.state))
          .range([0, innerHeight])
          .padding(0.1); //bar y axis scale

        const color = d3
          .scaleLinear()
          .domain([d3.max(data, (d) => d.data.value), 0])
          .nice()
          .range(ColorTheme[optionType]);

        mainChartColor = color;

        //layout
        const svg = container
          .selectAll("svg")
          .data(["svg"])
          .join("svg")
          .attr("width", "100%")
          .attr("viewBox", [0, 0, width, height])
          .attr("overflow", "visible");

        //Bar Chart
        const gBar = svg
          .selectAll("g.barchart")
          .data([0])
          .join("g")
          .attr("class", "barchart")
          .attr("transform", `translate(${margin.left},${margin.top})`)
          .call(bar);

        //Map shape
        const gMap = svg
          .selectAll("g.map")
          .data([0])
          .join("g")
          .attr("class", "map")
          .attr(
            "transform",
            `translate(${margin.left + barChartWidth},${margin.top})`
          )
          .call(map);

        //Map legend
        svg
          .selectAll("g.legend")
          .data([0])
          .join("g")
          .attr("class", "legend")
          .attr(
            "transform",
            `translate(${width - margin.right - 200},${margin.top})`
          )
          .call(legend);

        function map(g) {
          //map shape
          const path = g
            .selectAll("path")
            .data(data)
            .join("path")
            .attr("stroke", "white")
            .attr(
              "d",
              d3.geoPath().projection(
                d3
                  .geoMercator()
                  .center([81.9629, 22.5937]) //center of india
                  .translate([mapWidth / 2, innerHeight / 2]) //center of map on screen
                  .scale(1250)
              )
            );

          path.transition().attr("fill", (d) => color(d.data.value));

          //tooltip
          path.selectAll("title").remove();
          path
            .append("title")
            .text(
              (d) => d.data.state + "\n" + d.data.type + "\n" + d.data.value
            );

          //event
          path
            .on("mouseover", function (e, dd) {
              stateHighlight(dd.data.state);
            })
            .on("mouseout", function () {
              stateRecover();
            });
        }

        function bar(g) {
          const item = g.selectAll("g").data(data, (d) => d.data.state);

          const itemEnter = item
            .enter()
            .append("g")
            .attr(
              "transform",
              (d) => `translate(${0},${yScale(d.data.state)})`
            );

          itemEnter
            .append("rect")
            .attr("fill", (d) => color(d.data.value))
            .attr("stroke", "none")
            .attr("height", yScale.bandwidth())
            .attr("width", 0)
            .transition()
            .attrTween("width", function (d) {
              return d3.interpolate(0, xScale(d.data.value));
            })
            .on("interrupt cancel", function (d) {
              d3.select(this).attr("width", xScale(d.data.value));
            });

          itemEnter
            .append("text")
            .attr("class", "state")
            .attr("dominant-baseline", "middle")
            .attr("text-anchor", "end")
            .attr("fill", "grey")
            .attr("font-size", "0.7rem")
            .attr("cursor", "default")
            .attr("x", -5)
            .attr("y", yScale.bandwidth() / 2)
            .text((d) => d.data.state);

          itemEnter
            .append("text")
            .attr("class", "value")
            .attr("dominant-baseline", "middle")
            .attr("text-anchor", "start")
            .attr("fill", "grey")
            .attr("cursor", "default")
            .attr("font-size", "0.7rem")
            .attr("x", 5)
            .attr("y", yScale.bandwidth() / 2)
            .text((d) => d.data.value)
            .transition()
            .attrTween("x", function (d) {
              return d3.interpolate(5, xScale(d.data.value) + 5);
            })
            .on("interrupt cancel", function (d) {
              d3.select(this).attr("x", xScale(d.data.value) + 5);
            });

          const itemUpdate = item
            .transition()
            .attr(
              "transform",
              (d) => `translate(${0},${yScale(d.data.state)})`
            );

          itemUpdate
            .selectAll("rect")
            .transition()
            .on("interrupt cancel", function (d) {
              d3.select(this)
                .attr("fill", color(d.data.value))
                .attr("width", xScale(d.data.value))
                .attr("height", yScale.bandwidth());
            })
            .attr("fill", (d) => color(d.data.value))
            .attr("width", (d) => xScale(d.data.value))
            .attr("height", yScale.bandwidth());

          itemUpdate
            .selectAll("text.state")
            .transition()
            .on("interrupt cancel", function (d) {
              d3.select(this)
                .attr("y", yScale.bandwidth() / 2)
                .text(d.data.state);
            })
            .attr("y", yScale.bandwidth() / 2)
            .text((d) => d.data.state);

          itemUpdate
            .selectAll("text.value")
            .transition()
            .on("interrupt cancel", function (d) {
              d3.select(this)
                .attr("x", xScale(d.data.value) + 5)
                .attr("y", yScale.bandwidth() / 2)
                .text(d.data.value);
            })
            .attr("x", (d) => xScale(d.data.value) + 5)
            .attr("y", yScale.bandwidth() / 2)
            .text((d) => d.data.value);

          item.exit().remove();

          g.selectAll("*")
            .on("mouseover", function (e, dd) {
              stateHighlight(dd.data.state);
            })
            .on("mouseout", function () {
              stateRecover();
            });
        }

        function legend(g) {
          g.selectAll("*").remove();
          const rectWidth = 200;
          const rectHeight = 12;
          g.append("rect")
            .attr("fill", "url(#legend)")
            .attr("stroke", "none")
            .attr("width", rectWidth)
            .attr("height", rectHeight);

          g.append("defs")
            .append("linearGradient")
            .attr("id", "legend")
            .attr("x1", "0%")
            .attr("y1", "0%")
            .attr("x2", "100%")
            .attr("y2", "0%")
            .selectAll("stop")
            .data(color.range())
            .join("stop")
            .attr("offset", (d, i, a) => i / (a.length - 1))
            .attr("stop-color", (d) => d);

          const gTick = g
            .selectAll("g")
            .data([0, 0.25, 0.5, 0.75, 1])
            .join("g")
            .attr("transform", (d) => `translate(${rectWidth * d},${0})`);

          gTick
            .append("text")
            .attr("dominant-baseline", "hanging")
            .attr("text-anchor", "middle")
            .attr("fill", "grey")
            .attr("font-size", "0.6rem")
            .attr("y", rectHeight + 5)
            .text((d) =>
              d3.format(",")(
                (color.domain()[1] - color.domain()[0]) * d + color.domain()[0]
              )
            );

          gTick
            .append("line")
            .attr("stroke", "black")
            .attr("stroke-width", 1)
            .attr("y1", -3)
            .attr("y2", rectHeight + 3);
        }
      }

      function stateHighlight(targetState) {
        const color = mainChartColor;
        const gBar = d3.select(".main-chart").select(".barchart");
        const gMap = d3.select(".main-chart").select(".map");

        gBar
          .selectAll("rect")
          .filter((d) => d.data.state !== targetState)
          .transition()
          .attr("fill", "#E8E8E8");

        gBar
          .selectAll("text")
          .filter((d) => d.data.state !== targetState)
          .transition()
          .attr("fill", "#E8E8E8");

        gBar
          .selectAll("rect")
          .filter((d) => d.data.state == targetState)
          .transition()
          .attr("fill", (d) => color(d.data.value));

        gBar
          .selectAll("text")
          .filter((d) => d.data.state == targetState)
          .transition()
          .attr("fill", "black");

        gMap
          .selectAll("path")
          .filter((d) => d.data.state !== targetState)
          .transition()
          .attr("fill", "#E8E8E8");

        gMap
          .selectAll("path")
          .filter((d) => d.data.state == targetState)
          .transition()
          .attr("fill", (d) => color(d.data.value));

        //
        const donuts = d3.select(".donut1").selectAll("path");
        donuts
          .filter((d) => d.data.state !== targetState)
          .transition()
          .attrTween("fill", function () {
            const prev = d3.select(this).attr("fill");
            return d3.interpolate(prev, "#E8E8E8");
          })
          .on("interrupt cancel", function (d) {
            d3.select(this).attr("fill", "#E8E8E8");
          });

        //
        donuts
          .filter((d) => d.data.state == targetState)
          .transition()
          .attrTween("fill", function (d) {
            const prev = d3.select(this).attr("fill");
            return d3.interpolate(prev, donut1Color(d.value));
          })
          .on("interrupt cancel", function (d) {
            d3.select(this).attr("fill", (d) => donut1Color(d.value));
          });
      }

      function stateRecover() {
        const color = mainChartColor;
        const gBar = d3.select(".main-chart").select(".barchart");
        const gMap = d3.select(".main-chart").select(".map");
        gBar
          .selectAll("rect")
          .transition()
          .attr("fill", (d) => color(d.data.value));

        gBar.selectAll("text").transition().attr("fill", "grey");

        gMap
          .selectAll("path")
          .transition()
          .attr("fill", (d) => color(d.data.value))
          .attr("stroke", "white");

        const donuts = d3.select(".donut1").selectAll("path");
        //
        donuts
          .transition()
          .attr("fill", (d) => donut1Color(d.value))
          .on("interrupt cancel", function (d) {
            d3.select(this).attr("fill", (d) => donut1Color(d.value));
          });
      }
      function updateMainChartData(data, india, option) {
        const chosen = data
          .filter((row) => row.year == option.year)
          .map((row) => ({
            year: row.year,
            state: row.state,
            ...row.data.filter((d) => d.type == option.type)[0],
          }))
          .sort((a, b) => b.value - a.value);

        const matchList = {
          "Andaman and Nicobar": "A & N ISLANDS",
          "Dadra and Nagar Haveli": "D & N HAVELI",
          "Daman and Diu": "DAMAN & DIU",
          "Jammu and Kashmir": "JAMMU & KASHMIR",
          Orissa: "ODISHA",
          Uttaranchal: "UTTARAKHAND",
        };

        india.forEach((d, i) => {
          const geoStateName = d.properties["NAME_1"];
          var found = chosen.find(
            (c) => c.state.toUpperCase() == geoStateName.toUpperCase()
          );

          if (found == undefined) {
            found = chosen.find(
              (c) => c.state.toUpperCase() == matchList[geoStateName]
            );
          }
          d.data = found;
        });

        india.sort((a, b) => b.data.value - a.data.value);
        return india;
      }

      function parseData(raw) {
        const formatted = raw.map((d) => {
          //format
          const dd = {};
          dd.year = +d.Year;
          dd.state = d.STATE;
          dd.district = d.DISTRICT;
          Type.forEach((type) => {
            dd[type] = +d[type];
          });
          return dd;
        });

        return d3
          .rollups(
            formatted,
            (v) => {
              const dd = [];
              Type.forEach((type) => {
                dd.push({ type: type, value: d3.sum(v, (d) => d[type]) });
              });
              return { year: v[0].year, state: v[0].state, data: dd };
            },
            (d) => d.year + d.state
          )
          .map((d) => d[1]);
      }

      //
      function fetchData() {
        return Promise.all([d3.csv(csvUrl), d3.json(topojsonUrl)]);
      }